/*
 *  TestUtils.cpp
 *  Ottoman
 *
 *  Created by Jens Alfke on 9/12/09.
 *  Copyright 2009 Jens Alfke. All rights reserved.
 *  BSD-Licensed: See the file "LICENSE.txt" for details.
 */

#include "TestUtils.h"
#include "File.h"
#include <time.h>
#include <sys/time.h>
#include <stdio.h>

namespace Mooseyard {

    std::ostream& operator<< (std::ostream &out, const Blob &blob) {
        char str[blob.length+1];
        memcpy(str,blob.bytes,blob.length);
        str[blob.length] = 0;
        out << str;
        return out;
    }
    
    void shuffle(int a[], int n, unsigned seed) {
        if (seed==0) {
#ifdef BSD
            srandomdev();
#endif
            seed = random();
        }
        fprintf(stderr,"shuffle(n=%i, seed=%u)\n", n,seed);
        srandom(seed);
        for (int i=0; i<n-1; i++) {
            int j = i + (random() % (n-i));
            int temp = a[i];
            a[i] = a[j];
            a[j] = temp;
        }
    }
    
    char **sWords;
    int sNWords;
    
    void readWords() {
        if (!sWords) {
            sWords = new char*[250000];
            sNWords = 0;
            
            FILE *in = fopen("/usr/share/dict/words", "r");
            char word[4096];
            size_t wordLen;
            while (NULL != ::fgets(word, 4096, in)) {
                wordLen = ::strlen(word);
                if (word[wordLen-1]=='\n') {
		  word[wordLen-1] = '\0';
		  wordLen--;
		}

                if (wordLen == 0) continue;

                sWords[sNWords] = strdup(word); 
                //if( sNWords % 10000 == 0)
                //    printf("'%s' ... ", sWords[sNWords]->string());
                sNWords++;
            }
        }
    }
    
#pragma mark -
#pragma mark TIMER:
    
    Timer::Timer (const char *operation, int divisor) {
        _operation = operation;
        _divisor = divisor;
        _cpuTime = clock();
        _time = now();
    }
    
    Timer::~Timer() {
        double elapsedCPU = (clock() - _cpuTime) / 1.0e6;
        double elapsed = now() - _time;
        printf("### %s took %.6lf sec (used %.6lf sec CPU)", _operation, elapsed, elapsedCPU);
        if (_divisor > 1)
            printf(" ... per item: %.3lf usec, %.3lf usec CPU", elapsed/_divisor*1e6, elapsedCPU/_divisor*1e6);
        printf("\n");
    }
    
    double Timer::now() {
        struct timeval t;
        gettimeofday(&t,NULL);
        return (double)t.tv_sec + t.tv_usec/1.0e6;
    }
    
}


using namespace Mooseyard;

int main(int argc, char **argv) {
#if BSD
    srandomdev();
#endif
    try {
        ::testing::InitGoogleTest(&argc, argv);
        return RUN_ALL_TESTS();
    } catch (const File::Error &err) {
        fprintf(stderr, "\n*** File::Error thrown: %i/%s\n", err.code,err.message);
        throw;
    }
}
